#! /usr/bin/python


from twisted.internet import gtk2reactor
gtk2reactor.install()
from twisted.internet import reactor
from twisted.python import log

import petmail.crypto
petmail.crypto.EncryptionAgent = petmail.crypto.FakeEncryptionAgent

from petmail import agent, addressbook, message
from petmail.gui.ticketAgent import TicketAgent

import pygtk
pygtk.require("2.0")
import gtk, gtk.glade, gobject

import pickle, os, os.path

class MePane(object):
    def __init__(self, gui):
        self.gui = gui
        self.reverting = False

    def load(self, xml):
        g = xml.get_widget
        self.addresses_entry = g('my_addresses_entry')
        self.name_entry = g("my_name_entry")
        self.id_entry = g("my_ID_label")
        self.inlet_entry = g("inlet_entry")
        self.smtp_entry = g("smtp_entry")
        self.lookup_entry = g("addresslookup_entry")
        self.nntp_entry = g("nntp_entry")

        self.img_entry = g("my_IMG_entry")
        self.maxsize_entry = g("my_maxsize_entry")
        self.html_button = g("my_accept_html")
        self.ok_button = g("my_ok_button")
        self.cancel_button = g("my_cancel_button")
        self.ticket_urls = g("my_ticket_urls_entry")

        self.addresses_entry.connect("changed", self.dirty)
        self.name_entry.connect("changed", self.dirty)
        self.inlet_entry.connect("changed", self.dirty)
        self.smtp_entry.connect("changed", self.dirty)
        self.lookup_entry.connect("changed", self.dirty)
        self.nntp_entry.connect("changed", self.dirty)

        self.img_entry.connect("changed", self.dirty)
        self.maxsize_entry.connect("changed", self.dirty)
        self.html_button.connect("toggled", self.dirty)
        self.ticket_urls.connect("changed", self.dirty)

        xml.signal_connect("do_me_cancel", self.do_cancel)
        xml.signal_connect("do_me_ok", self.do_save)

    def dirty(self, *args):
        if self.reverting:
            return
        self.ok_button.set_sensitive(1)
        self.cancel_button.set_sensitive(1)

    def do_cancel(self, widget):
        #self.reverting = True
        self.populate()

    def do_save(self, widget):
        print "saving"
        me = self.gui.agent.me

        # addresses
        text = self.addresses_entry.get_text()
        pieces = text.split(",")
        new = []
        for i in pieces:
            i = i.strip()
            if i:
                new.append(i)
        me.addresses = new
        print new

        # name
        text = self.name_entry.get_text()
        text.strip()
        me.name = text

        # inlet
        inlet = self.inlet_entry.get_text()
        if inlet:
            me.setInlet(inlet)

        # other local parameters
        agent = self.gui.agent
        agent.setSMTP(self.smtp_entry.get_text())
        agent.setAddressLookup(self.lookup_entry.get_text())
        agent.setNNTP(self.nntp_entry.get_text())

        # our GenericPermission fields
        me.genericPermission.setMinIMG(self.img_entry.get_value())
        me.genericPermission.setMaxSize(self.maxsize_entry.get_value())
        # html: TODO
        urlString = self.ticket_urls.get_text()
        me.genericPermission.setTicketTakersFromString(self.gui.agent,
                                                       urlString)

        # save it all
        me.publish()
        self.gui.scheduleSave()

        self.ok_button.set_sensitive(0)
        self.cancel_button.set_sensitive(0)


    def populate(self):
        owner = self.gui.agent.me
        self.addresses_entry.set_text(", ".join(owner.addresses))
        if owner.name != None:
            self.name_entry.set_text(owner.name)
        else:
            self.name_entry.set_text("")
        self.id_entry.set_text(owner.ID)
        if owner.inlets:
            # TODO: this will be a fuzzy point: read vs write names for the
            # inlet
            self.inlet_entry.set_text(owner.inlets[0].encode())

        agent = self.gui.agent
        self.smtp_entry.set_text(agent.smtpScheme)
        self.lookup_entry.set_text(agent.getAddressLookup())
        if agent.nntpScheme:
            self.nntp_entry.set_text(agent.nntpScheme)

        self.img_entry.set_value(owner.genericPermission.minIMG)
        if owner.genericPermission.maxsize != None:
            self.maxsize_entry.set_value(owner.genericPermission.maxsize)
        else:
            self.maxsize_entry.set_value(99999)
        self.html_button.set_active(0)
        url_string = owner.genericPermission.getTicketTakersString()
        self.ticket_urls.set_text(url_string)
        self.reverting = False
        self.ok_button.set_sensitive(0)
        self.cancel_button.set_sensitive(0)

class AddressBookPane(object):
    def __init__(self, gui):
        self.gui = gui
        self.currentEntry = None

    def load(self, xml):
        # addressbook_tree, ID_entry, names_entry, petname_entry, IMT_entry,
        # maxsize_entry, accept_html
        # outbound_IMT, outbound_maxsize, outbound_html
        view = self.addressbook_tree = xml.get_widget("addressbook_tree")
        model = self.addressbook_list = gtk.ListStore(gobject.TYPE_STRING,
                                                      gobject.TYPE_PYOBJECT)
        view.set_model(model)
        view.append_column(gtk.TreeViewColumn("Name",
                                              gtk.CellRendererText(),
                                              text=0))
        sel = view.get_selection()
        sel.set_mode(gtk.SELECTION_SINGLE)
        sel.connect("changed", self.do_select)
        #self.populateAddressBook()
        g = xml.get_widget
        self.fields = {'petname': g("petname_entry"),
                       'ID': g("ID_label"),
                       'addrs': g("addresses_label"),
                       'name': g("name_label"),
                       'transport': g("transport_label"),

                       'i_IMT': g("IMT_entry"),
                       'i_size': g("maxsize_entry"),
                       'i_html': g("accept_html"),

                       'o_IMT': g("outbound_IMT"),
                       'o_size': g("outbound_maxsize"),
                       'o_html': g("outbound_html"),
                       'o_urls': g("outbound_ticket_urls_label"),
                       }
        self.fields['petname'].connect("activate", self.petname_entry_changed)
        self.do_select(sel)
        self.lookup_entry = g("lookup_entry")
        xml.signal_connect("do_lookup", self.do_lookup)

    def do_lookup(self, widget):
        addr = self.lookup_entry.get_text()
        if addr:
            d = self.gui.agent.retrieveAddress(addr)
            d.addCallback(self.lookup_complete)
            d.addCallback(log.err)
    def lookup_complete(self, record):
        book = self.gui.agent.getAddressBook(self.gui.owner)
        book.addRecord(record)
        self.populateAddressBook()
        
    def populateAddressBook(self):
        book = self.gui.agent.getAddressBook(self.gui.owner)
        self.addressbook_list.clear()
        for e in book.entries:
            iter = self.addressbook_list.append()
            self.addressbook_list.set(iter,
                                      0, e.getName(),
                                      1, e)

    def do_select(self, sel):
        model, iter = sel.get_selected()
        f = self.fields
        if not iter:
            self.currentEntry = None
            f['ID'].set_text("")
            f['addrs'].set_text("")
            f['name'].set_text("")
            f['petname'].set_text("")
            #f['i_IMT']
            #f['i_size']
            #f['i_html']
            f['o_IMT'].set_text("")
            f['o_size'].set_text("")
            f['o_html'].set_text("")
            f['o_urls'].set_text("")
            return # deselected
        entry = model.get_value(iter, 1)
        r = entry.idRecord
        assert(r)
        self.currentEntry = entry

        # names and such
        f['ID'].set_text(entry.idRecord.ID)
        f['addrs'].set_text(", ".join(r.addresses))
        if r.name != None:
            f['name'].set_text(r.name)
        else:
            f['name'].set_text("")
        if entry.petname != None:
            f['petname'].set_text(entry.petname)
        else:
            f['petname'].set_text("")
        t = entry.idRecord.transportString
        if t:
            f['transport'].set_text(t)
        else:
            f['transport'].set_text("<None>")
        # inbound
        p = entry.theirPermission
        if p:
            f['i_IMT'].set_value(p.minIMG)
            f['i_size'].set_value(p.maxsize)
            f['i_html'].set_active(1)
        else:
            # need to blank out the whole "our permissions" part or something
            pass

        # outbound
        p = entry.ourPermission
        if not p:
            p = entry.idRecord.genericPermission
        minIMG = p.minIMG
        if minIMG == None:
            minIMG = 0
        f['o_IMT'].set_text("%d seconds" % minIMG)
        maxsize = p.maxsize
        if maxsize == None:
            maxsize = 999999
        f['o_size'].set_text("%d kb" % maxsize)
        if 1:
            f['o_html'].set_text("Accepts HTML mail")
        else:
            f['o_html'].set_text("Does NOT accept HTML mail")
        f['o_urls'].set_text(", ".join(p.ticketURLs))

    def petname_entry_changed(self, widget):
        new_petname = widget.get_text()
        if self.currentEntry:
            if new_petname:
                self.currentEntry.petname = new_petname
            else:
                self.currentEntry.petname = None
            # update the list
            name = self.currentEntry.petname
            if name == None:
                name = self.currentEntry.idRecord.ID
            sel = self.addressbook_tree.get_selection()
            model, iter = sel.get_selected()
            model.set(iter, 0, self.currentEntry.getName())
            self.gui.scheduleSave()

    def updated(self, addrbook):
        self.populateAddressBook()
        self.gui.scheduleSave()

class SendPane(object):
    def __init__(self, gui):
        self.gui = gui
    def load(self, xml):
        self.fields = {'to': xml.get_widget("to_entry"),
                       'subject': xml.get_widget("subject_entry"),
                       'body': xml.get_widget("send_body"),
                       'send': xml.get_widget("send_button"),
                       'clear': xml.get_widget("clear_button"),
                       }
        self.fields['send'].connect('clicked', self.do_send)
        self.fields['clear'].connect('clicked', self.do_clear)
    def do_clear(self, button):
        self.fields['to'].set_text("")
        self.fields['subject'].set_text("")
        self.fields['body'].get_buffer().set_text("")
    def do_send(self, button):
        print "sending"
        to = self.fields['to'].get_text()
        names = []
        for name in to.split(","):
            names.append(name.strip())
        subject = self.fields['subject'].get_text()
        buf = self.fields['body'].get_buffer()
        start, end = buf.get_bounds()
        body = buf.get_text(start, end)
        #print type(body), body

        fullbody = "From: me\n" + "To: you\n" + "Subject: %s\n" % subject + \
                   "\n" + body
        if fullbody[-1] != "\n":
            fullbody += "\n"

        for name in names:
            m = message.Message(self.gui.agent)
            m.setSenderEntry(self.gui.owner)
            m.setMessageRecipient(name)
            m.addRFC822(fullbody)
            m.doDump = True
            m.send()

FROM = 0
SUBJECT = 1
MESSAGE = 2
class InboxPane(object):

    def __init__(self, gui):
        self.gui = gui

    def load(self, xml):
        g = xml.get_widget
        self.fields = {'from': g("from_label"),
                       'permission': g("from_permission_label"),
                       'body': g("inbox_body"),
                       }
        model = self.messages = gtk.ListStore(gobject.TYPE_STRING, # from
                                              gobject.TYPE_STRING, # subject
                                              gobject.TYPE_PYOBJECT, # msg
                                              )
        view = g("inbox_tree")
        view.set_model(model)
        view.append_column(gtk.TreeViewColumn("From",
                                              gtk.CellRendererText(),
                                              text=FROM))
        view.append_column(gtk.TreeViewColumn("Subject",
                                              gtk.CellRendererText(),
                                              text=SUBJECT))
        sel = view.get_selection()
        sel.set_mode(gtk.SELECTION_SINGLE)
        sel.connect("changed", self.do_message)

    def receive(self, msg, why):
        print "gui received"
        fromName = None
        msg.why = why
        if msg.senderEntry:
            fromName = msg.senderEntry.getName()
        if fromName is None:
            fromName = "??"
        self.messages.append((fromName, "<notyet>", msg))
        msg.body = msg.getNode('rfc822')['body']
        msg.doDump = True
        self.gui.scheduleSave()

    def do_message(self, sel):
        model, iter = sel.get_selected()
        f = self.fields
        if not iter:
            return
        msg = model.get_value(iter, MESSAGE)
        fromName = None
        if msg.senderEntry:
            fromName = msg.senderEntry.getName()
        if fromName is None:
            fromName = "??"
        f['from'].set_text(fromName)
        f['permission'].set_text(msg.why)
        f['body'].get_buffer().set_text(msg.body)

class SheepBox(object):
    def __init__(self, gui):
        self.gui = gui
    def load(self, xml):
        self.area = xml.get_widget("sheeparea")
        self.label = xml.get_widget("sheeplabel")
        self.entry = xml.get_widget("sheepentry")
        self.plantSheep()
        self.showEntry(0)

    def OLDmakeLabel(self, text):
        self.label.set_text(text)
        if self.boxContents == self.label:
            return
        if self.boxContents != None:
            self.boxContents.hide()
            self.box.remove(self.boxContents)
        self.box.add(self.label)
        self.boxContents = self.label
        self.boxContents.show_all()
    def OLDmakeEntry(self, instructions):
        self.vboxInstructions.set_text(instructions)
        if self.boxContents == self.vbox:
            return
        if self.boxContents != None:
            self.boxContents.hide()
            self.box.remove(self.boxContents)
        self.box.add(self.vbox)
        self.boxContents = self.vbox
        self.boxContents.show_all()

    def showEntry(self, show):
        if show:
            self.entry.show()
        else:
            self.entry.hide()

    def plantSheep(self):
        self.area.set_from_file("docs/sheep.gif")

class Gui(object):
    def __init__(self, agent):
        self.agent = agent
        self.owner = self.agent.me
        self.addrpane = AddressBookPane(self)
        self.mepane = MePane(self)
        self.sendpane = SendPane(self)
        self.inboxpane = InboxPane(self)
        self.sheepbox = SheepBox(self)
        self.saveTimer = None

    def scheduleSave(self):
        if self.saveTimer:
            self.saveTimer.cancel()
        self.saveTimer = reactor.callLater(5, self.do_save)

    def do_quit(self, widget):
        if self.saveTimer:
            self.saveTimer.cancel()
            self.saveTimer = None
            self.do_save()
        self.agent.stop()
    
    def loadXML(self):
        xml = gtk.glade.XML("petmail/gui/gui.glade")
        xml.signal_connect("do_quit", self.do_quit)
        self.addrpane.load(xml)
        self.mepane.load(xml)
        self.sendpane.load(xml)
        self.inboxpane.load(xml)
        self.sheepbox.load(xml)
        self.ticketAgents = [TicketAgent(self)]

    def do_save(self):
        if self.saveTimer:
            self.saveTimer = None
        if self.savefile:
            print "saving to", self.savefile
            tmpfile = self.savefile + ".tmp"
            try:
                pickle.dump(self.agent, open(tmpfile, "w"))
                os.rename(tmpfile, self.savefile)
            except (OSError, IOError), x:
                print "couldn't save:", x

class GuiAgent(agent.Agent):
    def __getstate__(self):
        d = agent.Agent.__getstate__(self)
        del d['gui']
        return d

    def getTicketAgents(self):
        return self.gui.ticketAgents
    def acceptRFC822(self, rm, why):
        return self.gui.inboxpane.receive(rm, why)
    def addressBookChanged(self, addrbook):
        print "addressBookChanged"
        return self.gui.addrpane.updated(addrbook)

def run(basedir):
    print "basedir", basedir
    savefile = os.path.join(basedir, "gui.pickle")
    if os.path.exists(savefile):
        print "loading"
        a = pickle.load(open(savefile, "r"))
    else:
        print "initializing"
        if not os.path.exists(basedir):
            os.mkdir(basedir)
        a = GuiAgent(basedir)
        a.init()
        
    gui = Gui(a)
    a.gui = gui
    gui.savefile = savefile
    gui.loadXML()
    gui.addrpane.populateAddressBook()
    gui.mepane.populate()
    a.run()

if __name__ == '__main__':
    import sys
    savefile = None
    if len(sys.argv) > 1:
        savefile = sys.argv[1]
    run(savefile)
