#! /usr/bin/python

from twisted.trial import unittest
from twisted.spread import pb
from twisted.internet import reactor

import petmail.crypto
petmail.crypto.EncryptionAgent = petmail.crypto.FakeEncryptionAgent

from petmail import util, ticket, ticketserver, crypto

class TestClient(ticketserver.PBTicketClient):
    def displayImage(self, image):
        self.sendResponse(image)

class Test(unittest.TestCase):
    def setUp(self):
        self.crypto = cagent = crypto.FakeEncryptionAgent()
        privkey = cagent.generateKey("ticketserver")
        server = ticketserver.PBTicketServer(None, privkey, fake=True)
        factory = pb.PBServerFactory(server, unsafeTracebacks=True)
        p = reactor.listenTCP(0, factory)
        portnum = p.getHost()[2]
        self.url = "PB:localhost:%d" % portnum
        server.url = self.url

    def testGetKey(self):
        d = ticketserver.PBTicketKeyClient(self.url).getKey()
        keydata = unittest.deferredResult(d)
        key = self.crypto.addPubkey(keydata)
        return key

    def testGetTicket(self):
        key = self.testGetKey()
        now = util.now()
        d = TestClient(self.url).getTicket()
        signed = unittest.deferredResult(d)
        sigID, plaintext = self.crypto.verify(signed)
        t = util.decode(plaintext)
        self.failUnlessEqual(sigID, key.getID())
        self.failUnlessEqual(t['URL'], self.url)
        self.assertApproximates(now, t['Timestamp'], 5)
        self.failUnless(t['TicketID'])

        taker = ticket.TicketTaker(self.url, None, key)
        self.failUnless(taker.checkTicket(sigID, t))
